@@ -1,6 +1,6 @@
 {- |
    Module      :  Text.ParserCombinators.Parsec.Rfc2822
-   Copyright   :  (c) 2012 Peter Simons
+   Copyright   :  (c) 2013 Peter Simons
    License     :  BSD3
 
    Maintainer  :  simons@cryp.to
@@ -10,8 +10,6 @@
    This module provides parsers for the grammar defined in
    RFC2822, \"Internet Message Format\",
    <http://www.faqs.org/rfcs/rfc2822.html>.
-
-   /Please note:/ The module is not particularly well tested.
 -}
 
 module Text.ParserCombinators.Parsec.Rfc2822 where
@@ -19,10 +17,14 @@ module Text.ParserCombinators.Parsec.Rfc2822 where
 import System.Time
 import Data.Char ( ord )
 import Data.List ( intercalate )
+import Data.Maybe ( catMaybes )
 import Control.Monad ( liftM )
 import Text.ParserCombinators.Parsec
 import Text.ParserCombinators.Parsec.Rfc2234 hiding ( quoted_pair, quoted_string )
 
+-- Customize hlint ...
+{-# ANN module "HLint: ignore Use camelCase" #-}
+
 -- * Useful parser combinators
 
 -- |Return @Nothing@ if the given parser doesn't match. This
@@ -80,7 +82,7 @@ specials        = oneOf "()<>[]:;@,.\\\""   <?> "one of ()<>[]:;@,.\\\""
 -- backslash and the actual content.
 
 quoted_pair     :: CharParser a String
-quoted_pair     = do { _ <- char '\\'; r <- text; return ['\\',r] }
+quoted_pair     = try obs_qp <|> do { _ <- char '\\'; r <- text; return ['\\',r] }
                   <?> "quoted pair"
 
 
@@ -260,8 +262,8 @@ date_time       = do wd <- option Monday (try (do wd <- day_of_week
                      return (CalendarTime y m d (tdHour td) (tdMin td) (tdSec td) 0 wd 0 "" z False)
                   <?> "date/time specification"
 
--- |This parser will match a 'day_name', optionally wrapped in folding
--- whitespace, or an 'obs_day_of_week' and return it's 'Day' value.
+-- |This parser matches a 'day_name' or an 'obs_day_of_week' (optionally
+-- wrapped in folding whitespace) and return its 'Day' value.
 
 day_of_week     :: CharParser a Day
 day_of_week     =     try (between (optional fws) (optional fws) day_name <?> "name of a day-of-the-week")
@@ -291,7 +293,7 @@ date            = do d <- day
                      return (y,m,d)
                   <?> "date specification"
 
--- |This parser will match a four digit number and return it's integer
+-- |This parser will match a four digit number and return its integer
 -- value. No range checking is performed.
 
 year            :: CharParser a Int
@@ -300,7 +302,7 @@ year            = do y <- manyN 4 digit
                   <?> "year"
 
 -- |This parser will match a 'month_name', optionally wrapped in
--- folding whitespace, or an 'obs_month' and return it's 'Month'
+-- folding whitespace, or an 'obs_month' and return its 'Month'
 -- value.
 
 month           :: CharParser a Month
@@ -326,11 +328,16 @@ month_name      =     do { try (caseString "Jan"); return January }
                   <|> do { caseString "Dec"; return December }
                   <?> "month name"
 
--- |Match either an 'obs_day', or a one or two digit number and return it.
+-- Internal helper function: match a 1 or 2-digit number (day of month).
+
+day_of_month    :: CharParser a Int
+day_of_month    = fmap read (manyNtoM 1 2 digit)
+
+-- |Match a 1 or 2-digit number (day of month), recognizing both
+-- standard and obsolete folding syntax.
 
 day             :: CharParser a Int
-day             =  try (do { optional fws; r <- manyNtoM 1 2 digit; return (read r :: Int) }) <|> obs_day
-                   <?> "day"
+day             = try obs_day <|> day_of_month <?> "day"
 
 -- |This parser will match a 'time_of_day' specification followed by a
 -- 'zone'. It returns the tuple (TimeDiff,Int) corresponding to the
@@ -354,7 +361,7 @@ time_of_day     = do h <- hour
                      return (TimeDiff 0 0 0 h m s 0)
                   <?> "time specification"
 
--- |This parser will match a two-digit number and return it's integer
+-- |This parser will match a two-digit number and return its integer
 -- value. No range checking is performed.
 
 hour            :: CharParser a Int
@@ -362,7 +369,7 @@ hour            = do r <- count 2 digit
                      return (read r :: Int)
                   <?> "hour"
 
--- |This parser will match a two-digit number and return it's integer
+-- |This parser will match a two-digit number and return its integer
 -- value. No range checking is performed.
 
 minute          :: CharParser a Int
@@ -370,7 +377,7 @@ minute          = do r <- count 2 digit
                      return (read r :: Int)
                   <?> "minute"
 
--- |This parser will match a two-digit number and return it's integer
+-- |This parser will match a two-digit number and return its integer
 -- value. No range checking takes place.
 
 second          :: CharParser a Int
@@ -445,11 +452,8 @@ angle_addr      = try (unfold (do _ <- char '<'
 -- semicolon. The found address(es) are returned - what may be none.
 -- Here is an example:
 --
--- >    parse group "" "my group: user1@example.org, user2@example.org;"
---
--- This input comes out as:
---
--- >    Right ["user1@example.org","user2@example.org"]
+-- >>> parse group "" "my group: user1@example.org, user2@example.org;"
+-- Right [NameAddr {nameAddr_name = Nothing, nameAddr_addr = "user1@example.org"},NameAddr {nameAddr_name = Nothing, nameAddr_addr = "user2@example.org"}]
 
 group           :: CharParser a [NameAddr]
 group           = do _ <- display_name
@@ -496,14 +500,14 @@ addr_spec       = do r1 <- local_part
 -- a 'dot_atom' or a 'quoted_string'.
 
 local_part      :: CharParser a String
-local_part      = dot_atom <|> quoted_string
+local_part      = try obs_local_part <|> dot_atom <|> quoted_string
                   <?> "address' local part"
 
 -- |Parse and return a \"domain part\" of an 'addr_spec'. That is either
 -- a 'dot_atom' or a 'domain_literal'.
 
 domain          :: CharParser a String
-domain          = dot_atom <|> domain_literal
+domain          = try obs_domain <|> dot_atom <|> domain_literal
                   <?> "address' domain part"
 
 -- |Parse a \"domain literal\". That is a \"@[@\" character, followed by
@@ -512,7 +516,7 @@ domain          = dot_atom <|> domain_literal
 
 domain_literal  :: CharParser a String
 domain_literal  = unfold (do _ <- char '['
-                             r <- many $ do { optional fws; dcontent }
+                             r <- many (optional fws >> dcontent)
                              optional fws
                              _ <- char ']'
                              return ("[" ++ concat r ++ "]"))
@@ -530,8 +534,8 @@ dcontent        = many1 dtext <|> quoted_pair
 
 dtext           :: CharParser a Char
 dtext           = no_ws_ctl
-                  <|> satisfy (\c -> ord c `elem` ([33..90] ++ [94,127]))
-                  <?> "character (excluding '[', ']', and '\\')"
+                  <|> satisfy (\c -> ord c `elem` ([33..90] ++ [94..126]))
+                  <?> "any ASCII character (excluding '[', ']', and '\\')"
 
 
 -- * Overall message syntax (section 3.5)
@@ -565,16 +569,10 @@ message         = do f <- fields
                      b <- option [] (do _ <- crlf; body)
                      return (Message f b)
 
--- |This parser will return a message body as specified by this RFC;
--- that is basically any number of 'text' characters, which may be
--- divided into separate lines by 'crlf'.
+-- |A message body is just an unstructured sequence of characters.
 
 body            :: CharParser a String
-body            = do r1 <- many (try (do line <- many text
-                                         eol <- crlf
-                                         return (line ++ eol)))
-                     r2 <- many text
-                     return (concat r1 ++ r2)
+body            = many anyChar
 
 
 -- * Field definitions (section 3.6)
@@ -774,12 +772,16 @@ no_fold_literal = do _ <- char '['
 
 -- ** Informational fields (section 3.6.5)
 
--- |Parse a \"@Subject:@\" header line and return it's contents verbatim.
+-- |Parse a \"@Subject:@\" header line and return its contents verbatim.
+-- Please note that all whitespace and/or comments are preserved, i.e.
+-- the result of parsing @\"Subject: foo\"@ is @\" foo\"@, not @\"foo\"@.
 
 subject         :: CharParser a String
 subject         = header "Subject" unstructured
 
--- |Parse a \"@Comments:@\" header line and return it's contents verbatim.
+-- |Parse a \"@Comments:@\" header line and return its contents verbatim.
+-- Please note that all whitespace and/or comments are preserved, i.e.
+-- the result of parsing @\"Comments: foo\"@ is @\" foo\"@, not @\"foo\"@.
 
 comments        :: CharParser a String
 comments        = header "Comments" unstructured
@@ -848,13 +850,14 @@ resent_msg_id   = header "Resent-Message-ID" msg_id
 -- ** Trace fields (section 3.6.7)
 
 return_path     :: CharParser a String
-return_path     = header "Return-Path:" path
+return_path     = header "Return-Path" path
 
 path            :: CharParser a String
-path            = unfold (    do _ <- char '<'
-                                 r <- choice [ try addr_spec, do { _ <- cfws; return [] } ]
-                                 _ <- char '>'
-                                 return ("<" ++ r ++ ">")
+path            = unfold ( try (do _ <- char '<'
+                                   r <- option "" addr_spec
+                                   _ <- char '>'
+                                   return ("<" ++ r ++ ">")
+                               )
                           <|> obs_path
                          )
                   <?> "return path spec"
@@ -1027,7 +1030,7 @@ obs_month       = between cfws cfws month_name <?> "month name"
 -- |Parse a 'day' but allow for the obsolete folding syntax.
 
 obs_day         :: CharParser a Int
-obs_day         = unfold day <?> "day"
+obs_day         = unfold day_of_month <?> "day"
 
 -- |Parse a 'hour' but allow for the obsolete folding syntax.
 
@@ -1067,33 +1070,22 @@ obs_zone        = choice [ mkZone "UT"  0
 
 -- * Obsolete Addressing (section 4.4)
 
--- |This parser will match the \"obsolete angle address\" syntax. This
--- construct used to be known as a \"route address\" in earlier RFCs.
--- There are two differences between this construct and the
--- 'angle_addr': For one - as usual -, the obsolete form allows for
--- more liberal insertion of folding whitespace and comments.
+-- |This parser matches the \"obsolete angle address\" syntax, a construct that
+-- used to be called \"route address\" in earlier RFCs. It differs from a
+-- standard 'angle_addr' in two ways: (1) it allows far more
+-- liberal insertion of folding whitespace and comments and (2) the address may
+-- contain a \"route\" (which this parser ignores):
 --
--- Secondly, and more importantly, angle addresses used to allow the
--- (optional) specification of a \"route\". The newer version does not.
--- Such a routing address looks like this:
---
--- >    <@example1.org,@example2.org:simons@example.org>
---
--- The parser will return a tuple that - in case of the above address -
--- looks like this:
---
--- >    (["example1.org","example2.org"],"simons@example.org")
---
--- The first part contains a list of hosts that constitute the route
--- part. This list may be empty! The second part of the tuple is the
--- actual 'addr_spec' address.
+-- >>> parse obs_angle_addr "" "<@example1.org,@example2.org:joe@example.org>"
+-- Right "<joe@example.org>"
 
 obs_angle_addr  :: CharParser a String
 obs_angle_addr  = unfold (do _ <- char '<'
                              _ <- option [] obs_route
                              addr <- addr_spec
                              _ <- char '>'
-                             return addr)  -- TODO: route is lost here.
+                             return ("<" ++ addr ++ ">") -- TODO: route is lost here.
+                         )
                   <?> "obsolete angle address"
 
 -- |This parser parses the \"route\" part of 'obs_angle_addr' and
@@ -1114,8 +1106,7 @@ obs_domain_list = do _ <- char '@'
                      r2 <- many (do _ <- cfws <|> string ","
                                     optional cfws
                                     _ <- char '@'
-                                    r <- domain
-                                    return r)
+                                    domain)
                      return (r1 : r2)
                     <?> "route of an obsolete angle address"
 
@@ -1149,24 +1140,24 @@ obs_domain      = do r1 <- atom
 -- commas. But you may have multiple consecutive commas without giving
 -- a 'mailbox'. You may also have a valid 'obs_mbox_list' that
 -- contains /no/ 'mailbox' at all. On the other hand, you /must/ have
--- at least one comma.
+-- at least one comma. The following example is valid:
 --
--- So, this input is perfectly valid:
+-- >>> parse obs_mbox_list "" ","
+-- Right []
 --
--- >    ","
+-- But this one is not:
 --
--- But this one is - contrary to all intuition - not:
---
--- >    "simons@example.org"
---
--- Strange, isn't it?
+-- >>> parse obs_mbox_list "" "joe@example.org"
+-- Left (line 1, column 16):
+-- unexpected end of input
+-- expecting obsolete syntax for a list of mailboxes
 
 obs_mbox_list   :: CharParser a [NameAddr]
 obs_mbox_list   = do r1 <- many1 (try (do r <- maybeOption mailbox
                                           _ <- unfold $ char ','
                                           return r))
                      r2 <- maybeOption mailbox
-                     return [x | Just x <- r1 ++ [r2]]
+                     return (catMaybes (r1 ++ [r2]))
                   <?> "obsolete syntax for a list of mailboxes"
 
 -- |This parser is identical to 'obs_mbox_list' but parses a list of
@@ -1182,7 +1173,7 @@ obs_addr_list   = do r1 <- many1 (try (do r <- maybeOption address
                                           optional cfws
                                           return r))
                      r2 <- maybeOption address
-                     return (concat [x | Just x <- r1 ++ [r2]])
+                     return (concat (catMaybes (r1 ++ [r2])))
                   <?> "obsolete syntax for a list of addresses"
 
 
@@ -1386,7 +1377,7 @@ obs_resent_reply = obs_header "Resent-Reply-To" address_list
 
 -- ** Obsolete trace fields (section 4.5.7)
 
-obs_return      :: CharParser a [Char]
+obs_return      :: CharParser a String
 obs_return       = obs_header "Return-Path" path
 
 obs_received    :: CharParser a [(String, String)]
